@@ -15,65 +15,61 @@
 # limitations under the License.
 #
 
-from warnings import warn
-from pyspark import keyword_only
 from ai.h2o.sparkling import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
-from ai.h2o.sparkling.ml.algos.H2OSupervisedAlgorithm import H2OSupervisedAlgorithm
-from ai.h2o.sparkling.ml.params.H2OAutoMLParams import H2OAutoMLParams
-from ai.h2o.sparkling.ml.params.H2OCommonParams import H2OCommonParams
-from ai.h2o.sparkling.ml.algos.H2OAutoMLExtras import H2OAutoMLExtras
+from ai.h2o.sparkling.ml.algos.H2OSupervisedAlgoBase import H2OSupervisedAlgoBase
+from ai.h2o.sparkling.ml.params import H2OAutoMLParams
+from pyspark import keyword_only
+from pyspark.sql import SparkSession
+from pyspark.sql.dataframe import DataFrame
 
 
-class H2OAutoML(H2OAutoMLParams, H2OCommonParams, H2OSupervisedAlgorithm, H2OAutoMLExtras):
+class H2OAutoML(H2OAutoMLParams, H2OSupervisedAlgoBase):
 
     @keyword_only
     def __init__(self,
-                 columnsToCategorical=[],
-                 withContributions=False,
-                 withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
-                 convertInvalidNumbersToNa=False,
-                 detailedPredictionCol="detailed_prediction",
-                 validationDataFrame=None,
-                 featuresCols=[],
-                 predictionCol="prediction",
-                 convertUnknownCategoricalLevelsToNa=False,
-                 splitRatio=1.0,
-                 ignoredCols=None,
-                 monotoneConstraints={},
-                 withStageResults=False,
-                 withDetailedPredictionCol=True,
+                 ignoredCols=[],
+                 includeAlgos=["GLM", "DRF", "GBM", "DeepLearning", "StackedEnsemble", "XGBoost"],
+                 excludeAlgos=[],
                  projectName=None,
-                 nfolds=5,
+                 maxRuntimeSecs=0.0,
+                 stoppingRounds=3,
+                 stoppingTolerance=0.001,
+                 stoppingMetric="AUTO",
+                 sortMetric="AUTO",
                  balanceClasses=False,
                  classSamplingFactors=None,
                  maxAfterBalanceSize=5.0,
                  keepCrossValidationPredictions=False,
                  keepCrossValidationModels=False,
-                 keepCrossValidationFoldAssignment=False,
-                 exportCheckpointsDir=None,
+                 maxModels=0,
                  labelCol="label",
                  foldCol=None,
                  weightCol=None,
-                 sortMetric="AUTO",
+                 offsetCol=None,
+                 splitRatio=1.0,
                  seed=-1,
-                 maxModels=0,
-                 maxRuntimeSecs=0.0,
-                 maxRuntimeSecsPerModel=0.0,
-                 stoppingRounds=3,
-                 stoppingMetric="AUTO",
-                 stoppingTolerance=-1.0,
-                 excludeAlgos=None,
-                 includeAlgos=["GLM", "DRF", "GBM", "DeepLearning", "StackedEnsemble", "XGBoost"],
-                 exploitationRatio=0.0):
+                 nfolds=5,
+                 allStringColumnsToCategorical=True,
+                 columnsToCategorical=[],
+                 predictionCol="prediction",
+                 detailedPredictionCol="detailed_prediction",
+                 withDetailedPredictionCol=False,
+                 featuresCols=[],
+                 convertUnknownCategoricalLevelsToNa=False,
+                 convertInvalidNumbersToNa=False,
+                 namedMojoOutputColumns=True,
+                 monotoneConstraints={},
+                 withContributions=False):
         Initializer.load_sparkling_jar()
         super(H2OAutoML, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.H2OAutoML", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
-
-        if 'interactionPairs' in kwargs:
-            warn("Interaction pairs are not supported!")
         self._set(**kwargs)
-        self._transfer_params_to_java()
+
+    def getLeaderboard(self, *extraColumns):
+        if len(extraColumns) == 1 and isinstance(extraColumns[0], list):
+            extraColumns = extraColumns[0]
+        leaderboard_java = self._java_obj.getLeaderboard(extraColumns)
+        return DataFrame(leaderboard_java, SparkSession.builder.getOrCreate()._wrapped)
