@@ -7,13 +7,12 @@ Rapids expressions. These are helper classes for H2OFrame.
 """
 from __future__ import division, print_function, absolute_import, unicode_literals
 
-from collections import Iterable, OrderedDict
+from collections import OrderedDict
 import copy
 import gc
 import inspect
 import math
 import time
-import traceback
 import numbers
 
 import tabulate
@@ -89,23 +88,29 @@ class ExprNode(object):
     def _eager_frame(self):
         if not self._cache.is_empty(): return
         if self._cache._id is not None: return  # Data already computed under ID, but not cached locally
-        self._eval_driver(True)
+        self._eval_driver('frame')
 
     def _eager_scalar(self):  # returns a scalar (or a list of scalars)
         if not self._cache.is_empty():
             assert self._cache.is_scalar()
             return self
         assert self._cache._id is None
-        self._eval_driver(False)
+        self._eval_driver('scalar')
         assert self._cache._id is None
         assert self._cache.is_scalar()
         return self._cache._data
 
     def _eager_map_frame(self):  # returns a scalar (or a list of scalars)
-        self._eval_driver(False)
+        self._eval_driver('scalar')
         return self._cache
 
     def _eval_driver(self, top):
+        """
+        :param top: if this is a top expression (providing a final result),
+            then specifies the expected result type (accepted values = ['frame', 'scalar']),
+            or None if no object creation is expected.
+        :return: self expr
+        """
         exec_str = self._get_ast_str(top)
         res = ExprNode.rapids(exec_str)
         if 'scalar' in res:
@@ -138,7 +143,7 @@ class ExprNode(object):
     # Global timezone change is eager, so the time parse as to occur in the correct order relative to
     # the timezone change, so cannot be lazy.
     #
-    def _get_ast_str(self, top):
+    def _get_ast_str(self, top=None):
         if not self._cache.is_empty():  # Data already computed and cached; could a "false-like" cached value
             return str(self._cache._data) if self._cache.is_scalar() else self._cache._id
         if self._cache._id is not None:
@@ -149,16 +154,21 @@ class ExprNode(object):
         def is_ast_expr(ref):
             return isinstance(ref, list) and any(map(lambda r: isinstance(r, ASTId), ref))
 
+        def is_debug_ref(ref):
+            # if ref is a dict, then it is a `locals` scope: most of those are added in debug mode.
+            # However, keeping it if this scope refers to an H2OFrame (has `_ex` entry)
+            return isinstance(ref, dict) and '_ex' not in ref
+
         referrers = gc.get_referrers(self)
         # removing frames from the referrers to get a consistent behaviour accross Py versions
         #  as stack frames don't appear in the referrers from Py 3.7.
         # also removing the AST expressions built in astfun.py
         #  as they keep a reference to self if the lambda itself is using a free variable.
-        proper_ref = [r for r in referrers if not (inspect.isframe(r) or is_ast_expr(r))]
+        proper_ref = [r for r in referrers if not (inspect.isframe(r) or is_ast_expr(r) or is_debug_ref(r))]
         ref_cnt = len(proper_ref)
         del referrers, proper_ref
         # if this self node is referenced by at least one other node (nested expr), then create a tmp frame
-        if top or ref_cnt > 1:
+        if top == 'frame' or (not top and ref_cnt > 1):
             self._cache._id = _py_tmp_key(append=h2o.connection().session_id)
             exec_str = "(tmp= {} {})".format(self._cache._id, exec_str)
         return exec_str
@@ -168,7 +178,7 @@ class ExprNode(object):
         if arg is None:
             return "[]"  # empty list
         if isinstance(arg, ExprNode):
-            return arg._get_ast_str(False)
+            return arg._get_ast_str()
         if isinstance(arg, ASTId):
             return str(arg)
         if isinstance(arg, (list, tuple, range)):
